@@ -8,6 +8,7 @@ with OSC Tiny.
 
 .. versionadded:: 0.4.0
 """
+import typing
 from base64 import b64decode
 from bz2 import decompress
 from configparser import ConfigParser, NoSectionError
@@ -17,11 +18,12 @@ import warnings
 
 try:
     from osc import conf as _conf
+    from osc.oscerr import ConfigError, ConfigMissingApiurl
 except ImportError:
     _conf = None
 
 
-def get_config_path():
+def get_config_path() -> Path:
     """
     Return path of ``osc`` configuration file
 
@@ -43,7 +45,9 @@ def get_config_path():
     raise FileNotFoundError("No `osc` configuration file found")
 
 
-def get_credentials(url=None):
+# pylint: disable=too-many-branches
+def get_credentials(url: typing.Optional[str] = None) \
+        -> typing.Tuple[str, str, typing.Optional[Path]]:
     """
     Get credentials for Build Service instance identified by ``url``
 
@@ -56,26 +60,31 @@ def get_credentials(url=None):
 
     :param str url: URL of Build Service instance (including schema). If not specified, the value
                     from the ``apiurl`` parameter in the config file will be used.
-    :return: (username, password)
+    :return: (username, password, SSH private key path)
     :raises ValueError: if config provides no credentials
     """
     if _conf is not None:
-        # pylint: disable=protected-access
-        parser = _conf.get_configParser()
         try:
+            _conf.get_config()
             if url is None:
-                url = parser["general"].get("apiurl", url)
-            cred_mgr = _conf._get_credentials_manager(url, parser)
-            username = _conf._extract_user_compat(parser, url, cred_mgr)
-        except (KeyError, NoSectionError) as error:
-            raise ValueError("`osc` config does not provide the default API URL") from error
+                # get the default api url from osc's config
+                url = _conf.config["apiurl"]
+            # and now fetch the options for that particular url
+            api_config = _conf.get_apiurl_api_host_options(url)
+            username = api_config["user"]
+            password = api_config["pass"]
+            sshkey = Path(api_config["sshkey"]) if api_config["sshkey"] else None
+        except (ConfigError, ConfigMissingApiurl) as error:
+            if isinstance(error, ConfigError):
+                raise ValueError("`osc` config was not found.") from error
+            # this is the case of ConfigMissingApiurl
+            raise ValueError("`osc` config has no options for URL {}".format(url)) from error
 
         if not username:
             raise ValueError("`osc` config provides no username for URL {}".format(url))
-        password = cred_mgr.get_password(url, username, defer=False)
         if not password:
             raise ValueError("`osc` config provides no password for URL {}".format(url))
-        return username, password
+        return username, password, sshkey
 
     warnings.warn("`osc` is not installed. Not all configuration backends of `osc` will be "
                   "available.")
@@ -104,4 +113,8 @@ def get_credentials(url=None):
     if not password:
         raise ValueError("`osc` config provides no password for URL {}".format(url))
 
-    return username, password
+    sshkey = parser[url].get("sshkey", None)
+    if sshkey:
+        sshkey = Path(sshkey)
+
+    return username, password, sshkey
